/*
<DEPENDENCIES>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
var staticMarkerManager = new Array;
MarkerManager_getInstance = function( sId )
{
  var Instance = null;
  if( staticMarkerManager[sId] )
  {
    Instance = staticMarkerManager[sId];
  }
  return Instance;
}
function MarkerManager( sId, sBgColor, sFgColor )
{
  if( sId )
  {
    this.init(sId, sBgColor, sFgColor);
  }
}
MarkerManager.prototype.init = function( sId, sBgColor, sFgColor )
{
  ;
  staticMarkerManager[sId] = this;
  this.sId = sId;
  this.sBgColor = sBgColor;
  this.sFgColor = sFgColor;
  this.bMarkable = false;
  this.aMarkableAreas = new Array();
  this.sMarkableCursor = 'pointer';
  this.sNotMarkableCursor = 'default';
  this.oMarkerBroker = null;
  this.startSelectionPoint = null;
  this.lastSelectionRange = null;
  this.aSelectedSelectionRanges = new Array();
  var me = this;
  function eventHandlerCallback( event )
  {
    me.eventHandler(event);
  }
  this.eventHandlerCallback = eventHandlerCallback;
}
MarkerManager.prototype.setMarkerBroker = function( oMarkerBroker )
{
  this.oMarkerBroker = oMarkerBroker;
  this.oMarkerBroker.addSetEnableMarkableHandle(this.getSetEnableMarkableAction());
  this.oMarkerBroker.addSetDisableMarkableHandle(this.getSetDisableMarkableAction());
}
MarkerManager.prototype.start = function()
{
  if( this.oMarkerBroker )
  {
    if( this.aMarkableAreas.length > 0 )
    {
      Browser.addEventListenerToDocument(Browser.eventNames.MOUSEDOWN, this.eventHandlerCallback);
      Browser.addEventListenerToDocument(Browser.eventNames.CLICK, this.eventHandlerCallback);
      this.oMarkerBroker.activateMarker();
    }
    else
    {
      this.oMarkerBroker.deactivateMarker();
    }
  }
}
MarkerManager.prototype.setSelectableAreasArray = function( aDomIdArray )
{
  ;
  function MarkableArea( sDomId, oNode, aSelectableSpans )
  {
    this.sDomId = sDomId;
    this.oNode = oNode;
    this.aSelectableSpans = aSelectableSpans;
  }
  function Counter( nCount )
  {
    this.nCount = nCount;
  }
  var textHandler = this.getTextHandler();
  for( var i = 0; i < aDomIdArray.length; i++ )
  {
    var node = Browser.getElementById(aDomIdArray[i]);
    if( node )
    {
      node.markableArea = this.aMarkableAreas.length;
      this.aMarkableAreas.push(new MarkableArea(
              aDomIdArray[i],
              node,
              this.createSelectableSpans(node, this.aMarkableAreas.length, new Counter(0), textHandler)));
    }
  }
}
MarkerManager.prototype.createSelectableSpans = function( node, markableArea, counter, textHandler )
{
  ;
  var aTextSpans = new Array;
  if( node.nodeType == 3 )
  {
    var fragment = document.createElement('span');
    var text = node.data;
    textHandler(aTextSpans, markableArea, fragment, text, counter);
    node.parentNode.replaceChild(fragment, node);
  }
  else
  {
    for( var i = 0; i < node.childNodes.length; i++ )
    {
      var returnArray = this.createSelectableSpans(node.childNodes[i], markableArea, counter, textHandler);
      aTextSpans = aTextSpans.concat(returnArray);
    }
  }
  return aTextSpans;
}
MarkerManager.prototype.getTextHandler = function()
{
  ;
  var charset = Browser.getCharacterSet();
  if( !charset )
  {
    ;
    return this.handleTextDefault;
  }
  charset = charset.toLowerCase();
  switch( charset )
          {
    case 'utf-8':
    case 'utf8':
    case 'utf-16':
    case 'utf16':
      ;
      return this.handleTextUnicode;
    default:
      ;
      return this.handleTextDefault;
  }
}
MarkerManager.prototype.handleTextDefault = function( aTextSpans, markableArea, fragment, text, counter )
{
  ;
  for( var i = 0; i < text.length; i++ )
  {
    counter.nCount++;
    var span = document.createElement('span');
    aTextSpans.push(span);
    var textNode = document.createTextNode(text.charAt(i));
    span.appendChild(textNode);
    span.spanOffset = counter.nCount;
    span.markableArea = markableArea;
    span.nodeIsMarkable = true;
    fragment.appendChild(span);
  }
}
MarkerManager.prototype.handleTextUnicode = function( aTextSpans, markableArea, fragment, text, counter )
{
  ;
  for( var i = 0; i < text.length; i++ )
  {
    counter.nCount++;
    var span = document.createElement('span');
    aTextSpans.push(span);
    var c = text.charCodeAt(i);
    var nodeText = text.charAt(i);
    if( i < text.length - 1 )
    {
      var nextChar = text.charCodeAt(i + 1);
      while( ((nextChar > 767) && (nextChar < 880))
              || ((nextChar > 7615) && (nextChar < 7680))
              || ((nextChar > 8399) && (nextChar < 8448)) )
      {
        ;
        nodeText += text.charAt(++i);
      }
    }
    var textNode = document.createTextNode(nodeText);
    span.appendChild(textNode);
    span.spanOffset = counter.nCount;
    span.markableArea = markableArea;
    span.nodeIsMarkable = true;
    fragment.appendChild(span);
  }
}
MarkerManager.prototype.setMarkable = function( bMarkable )
{
  ;
  this.bMarkable = bMarkable;
  for( var i = 0; i < this.aMarkableAreas.length; i++ )
  {
    if( this.bMarkable )
    {
      this.aMarkableAreas[i].oNode.style.cursor = this.sMarkableCursor;
    }
    else
    {
      this.aMarkableAreas[i].oNode.style.cursor = this.sNotMarkableCursor;
    }
  }
}
MarkerManager.prototype.setMarkableCursor = function( sMarkableCursor )
{
  ;
  this.sMarkableCursor = sMarkableCursor;
}
MarkerManager.prototype.setNotMarkableCursor = function( sNotMarkableCursor )
{
  ;
  this.sNotMarkableCursor = sNotMarkableCursor;
}
MarkerManager.prototype.getSetEnableMarkableAction = function()
{
  return new ObjectMethodAction(this, "setMarkable", true);
}
MarkerManager.prototype.getSetDisableMarkableAction = function()
{
  return new ObjectMethodAction(this, "setMarkable", false);
}
MarkerManager.prototype.eventHandler = function( event )
{
  ;
  switch( event.type )
          {
    case 'mousedown':
      var originalTarget = Browser.getOriginalTargetFromEvent(event)
      if( this.bMarkable && originalTarget.nodeIsMarkable )
      {
        ;
        Browser.addEventListenerToDocument(Browser.eventNames.MOUSEMOVE, this.eventHandlerCallback)
        Browser.addEventListenerToDocument(Browser.eventNames.MOUSEUP, this.eventHandlerCallback)
        this.startSelectionPoint = new SelectionPoint(
                originalTarget.markableArea,
                originalTarget.spanOffset
                )
      }
      break;
    case 'mouseup':
      window.setTimeout('MarkerManager_getInstance(\'' + this.sId + '\').mouseupHandlerCallback();', 10);
      Browser.removeEventListenerFromDocument(Browser.eventNames.MOUSEMOVE, this.eventHandlerCallback)
      Browser.removeEventListenerFromDocument(Browser.eventNames.MOUSEUP, this.eventHandlerCallback)
      break;
    case 'mousemove':
      this.mousemoveHandler(event);
      break;
    case 'click':
      if( this.bMarkable )
      {
        var originalTarget = Browser.getOriginalTargetFromEvent(event)
        var selectionPoint = new SelectionPoint(
                originalTarget.markableArea,
                originalTarget.spanOffset
                );
        var tmpSelectedSelectionRanges = new Array();
        for( var i = 0; i < this.aSelectedSelectionRanges.length; i++ )
        {
          if( this.aSelectedSelectionRanges[i].isPointInRange(selectionPoint) )
          {
            this.colorizeSelectionRange(this.aSelectedSelectionRanges[i], false);
          }
          else
          {
            tmpSelectedSelectionRanges.push(this.aSelectedSelectionRanges[i])
          }
        }
        this.aSelectedSelectionRanges = tmpSelectedSelectionRanges;
      }
      break;
  }
}
MarkerManager.prototype.mouseupHandlerCallback = function()
{
  ;
  if( this.lastSelectionRange )
  {
    this.aSelectedSelectionRanges.push(this.lastSelectionRange);
    this.lastSelectionRange = null;
    this.startSelectionPoint = null;
  }
}
MarkerManager.prototype.mousemoveHandler = function( event )
{
  ;
  var originalTarget = Browser.getOriginalTargetFromEvent(event)
  var selectionPoint = new SelectionPoint(
          originalTarget.markableArea,
          originalTarget.spanOffset
          );
  if( this.lastSelectionRange != null )
  {
    this.colorizeSelectionRange(this.lastSelectionRange, false);
  }
  if( selectionPoint.markableArea == this.startSelectionPoint.markableArea )
  {
    var tmpSelectedSelectionRanges = new Array();
    for( var i = 0; i < this.aSelectedSelectionRanges.length; i++ )
    {
      if( this.aSelectedSelectionRanges[i].isPointInRange(selectionPoint) )
      {
        this.colorizeSelectionRange(this.aSelectedSelectionRanges[i], false);
      }
      else
      {
        tmpSelectedSelectionRanges.push(this.aSelectedSelectionRanges[i])
      }
    }
    this.aSelectedSelectionRanges = tmpSelectedSelectionRanges;
    if( selectionPoint.spanOffset >= this.startSelectionPoint.spanOffset )
    {
      this.lastSelectionRange = new SelectionRange(
              this.startSelectionPoint,
              selectionPoint
              );
    }
    else
    {
      this.lastSelectionRange = new SelectionRange(
              selectionPoint,
              this.startSelectionPoint
              );
    }
    this.colorizeSelectionRange(this.lastSelectionRange, true);
  }
}
MarkerManager.prototype.colorizeSelectionRange = function( selectionRange, bColorize )
{
  ;
  for( var i = selectionRange.startPoint.spanOffset; i <= selectionRange.stopPoint.spanOffset; i++ )
  {
    var node = this.aMarkableAreas[selectionRange.startPoint.markableArea].aSelectableSpans[i - 1];
    if( ! bColorize )
    {
      if( Browser.getId() == 'moz' )
      {
        node.style.background = 'inherit';
        node.style.color = 'inherit';
      }
      else
      {
        if( node.style )
        {
          node.style.background = '';
          node.style.color = '';
        }
      }
    }
    else
    {
      if( node.style )
      {
        node.style.background = this.sBgColor;
        node.style.color = this.sFgColor;
      }
    }
  }
}
MarkerManager.prototype.getResetAllAction = function()
{
  return new ObjectMethodAction(this, 'resetAll');
}
MarkerManager.prototype.resetAll = function()
{
  ;
  for( var i = 0; i < this.aSelectedSelectionRanges.length; i++ )
  {
    this.colorizeSelectionRange(this.aSelectedSelectionRanges[i], false);
  }
  this.aSelectedSelectionRanges = new Array();
}
function SelectionPoint( markableArea, spanOffset )
{
  this.markableArea = markableArea;
  this.spanOffset = spanOffset;
}
function SelectionRange( startPoint, stopPoint )
{
  this.startPoint = startPoint;
  this.stopPoint = stopPoint;
}
SelectionRange.prototype.isPointInRange = function( oPoint )
{
  var bIsInRange = false;
  if( this.startPoint.markableArea == oPoint.markableArea )
  {
    if( oPoint.spanOffset >= this.startPoint.spanOffset )
    {
      if( oPoint.spanOffset <= this.stopPoint.spanOffset )
      {
        bIsInRange = true;
      }
    }
  }
  return bIsInRange;
}
