/*
<DEPENDENCIES>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Snapshot</DEPENDS>
    <DEPENDS>Delta</DEPENDS>
    <DEPENDS>Alteration</DEPENDS>
</DEPENDENCIES>
*/
function AlterableWidget( sId, sWidgetDomId, bAlterLeft, bAlterTop, bAlterWidth, bAlterHeight )
{
  if( sId )
  {
    this.init(sId, sWidgetDomId, bAlterLeft, bAlterTop, bAlterWidth, bAlterHeight);
  }
}
AlterableWidget.prototype.init = function( sId, sWidgetDomId, bAlterLeft, bAlterTop, bAlterWidth, bAlterHeight )
{
  ;
  this.sId = sId;
  this.widgetDomId = sWidgetDomId;
  this.ownState = null;
  this.snapshotCount = 0;
  this.aAlterations = new Array();
  if( bAlterLeft )
    this.aAlterations.push(new AlterationLeft(this.sId + '_alterLeft'));
  if( bAlterTop )
    this.aAlterations.push(new AlterationTop(this.sId + '_alterTop'));
  if( bAlterWidth )
    this.aAlterations.push(new AlterationWidth(this.sId + '_alterWidth'));
  if( bAlterHeight )
    this.aAlterations.push(new AlterationHeight(this.sId + '_alterHeight'));
  this.aResetHandles = new Array();
  this.aAlteredStateHandles = new Array();
  this.aAlteredDeltaHandles = new Array();
}
AlterableWidget.prototype.getId = function()
{
  return this.sId;
}
AlterableWidget.prototype.registerAlterationHandler = function( alteration )
{
  if( alteration )
    this.aAlterations.push(alteration);
}
AlterableWidget.prototype.getRegisterAlterationHandlerAction = function()
{
  return new ObjectMethodAction(this, "registerAlterationHandler");
}
AlterableWidget.prototype.createSnapshot = function()
{
  return new Snapshot(this, this.snapshotCount++, document.getElementById(this.widgetDomId));
}
AlterableWidget.prototype.getCreateSnapshotAction = function()
{
  return new ObjectMethodAction(this, "createSnapshot");
}
AlterableWidget.prototype.reset = function( snapshot )
{
  if( snapshot )
  {
    for( var i = 0; i < this.aAlterations.length; i++ )
      this.aAlterations[i].reset(snapshot);
    AlterableWidget.notifyListeners(this.aResetHandles);
  }
}
AlterableWidget.prototype.getResetAction = function()
{
  return new ObjectMethodAction(this, "reset");
}
AlterableWidget.prototype.addResetHandle = function( action )
{
  if( action )
    this.aResetHandles.push(action);
}
AlterableWidget.prototype.isFeasible = function( snapshot, delta )
{
  if( snapshot && delta )
  {
    for( var i = 0; i < this.aAlterations.length; i++ )
    {
      var isFeasible = this.aAlterations[i].isFeasible(snapshot, delta);
      if( !isFeasible )
        return false; 
    }
    return true;
  }
}
AlterableWidget.prototype.getIsFeasibleAction = function()
{
  return new ObjectMethodAction(this, "isFeasible");
}
AlterableWidget.prototype.getFeasibleDelta = function( snapshot, delta )
{
  if( snapshot && delta )
  {
    var feasibleDelta = delta;
    for( var i = 0; i < this.aAlterations.length; i++ )
      feasibleDelta = this.aAlterations[i].getFeasibleDelta(snapshot, feasibleDelta);
    return feasibleDelta;
  }
}
AlterableWidget.prototype.getGetFeasibleDelta = function()
{
  return new ObjectMethodAction(this, "getFeasibleDelta");
}
AlterableWidget.prototype.alter = function( snapshot, delta )
{
  if( snapshot && delta )
  {
    var result = snapshot.copyNeutral();
    if( this.isFeasible(snapshot, delta) )
    {
      for( var i = 0; i < this.aAlterations.length; i++ )
        result.add(this.aAlterations[i].alter(snapshot, delta));
    }
    var ownDelta = result.clone();
    result.add(snapshot);
    this.ownState = result.clone();
    AlterableWidget.notifyListeners(this.aAlteredStateHandles, result);
    AlterableWidget.notifyListeners(this.aAlteredDeltaHandles, ownDelta);
  }
}
AlterableWidget.prototype.getAlterAction = function()
{
  return new ObjectMethodAction(this, "alter");
}
AlterableWidget.prototype.addAlteredStateHandle = function( action )
{
  if( action )
    this.aAlteredStateHandles.push(action);
}
AlterableWidget.prototype.handleAlteredDelta = function( deltaSnapshot )
{
  if( !this.ownState )
    this.ownState = this.createSnapshot();
  this.ownState.addPosition(deltaSnapshot);
  AlterableWidget.notifyListeners(this.aAlteredDeltaHandles, deltaSnapshot);
  AlterableWidget.notifyListeners(this.aAlteredStateHandles, this.ownState.clone());
}
AlterableWidget.prototype.getHandleAlteredDeltaAction = function( deltaSnapshot )
{
  return new ObjectMethodAction(this, "handleAlteredDelta", deltaSnapshot);
}
AlterableWidget.prototype.addAlteredDeltaHandle = function( action )
{
  if( action )
    this.aAlteredDeltaHandles.push(action);
}
AlterableWidget.notifyListeners = function( handles, param )
{
  for( var i = 0; i < handles.length; i++ )
  {
    handles[i].invoke(param);
  }
}
