/*
<DEPENDENCIES>
  <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function ConstructionArea( sId, sWidgetDomId, sImageDomId )
{
  if( sId )
  {
    this.init(sId, sWidgetDomId, sImageDomId);
  }
}
ConstructionArea.prototype.init = function( sId, sWidgetDomId, sImageDomId )
{
  ;
  this.sId = sId;
  this.widgetDomId = sWidgetDomId;
  this.nWidget = document.getElementById(this.widgetDomId);
  this.imageDomId = sImageDomId;
  this.nImage = document.getElementById(this.imageDomId);
  this.constructionModel = null;
  this.aClearedHandles = new Array();
  this.aElementCreatedHandles = new Array();
  this.aRootSetHandles = new Array();
  this.aElementAddedHandles = new Array();
  this.aElementRemovedHandles = new Array();
  this.aElementFoundHandles = new Array();
  this.aImageSrcSetHandles = new Array();
  this.aMouseClickHandles = new Array();
  this.aMouseDropHandles = new Array();
  ConstructionArea.createMouseClickEventHandler(this);
}
ConstructionArea.prototype.getId = function()
{
  return this.sId;
}
ConstructionArea.prototype.start = function()
{
  Browser.addEventListenerToDomNode(this.nWidget, Browser.eventNames.CLICK, this.mouseClickHandler);
}
ConstructionArea.prototype.registerConstructionModel = function( constructionModel )
{
  this.constructionModel = constructionModel;
}
ConstructionArea.prototype.clear = function()
{
  this.constructionModel.clear();
  ConstructionArea.notifyListeners(this.aClearedHandles);
}
ConstructionArea.prototype.getClearAction = function()
{
  return new ObjectMethodAction(this, "clear");
}
ConstructionArea.prototype.addClearedHandle = function( action )
{
  if( action )
    this.aClearedHandles.push(action);
}
ConstructionArea.prototype.setRoot = function( modelElement )
{
  if( modelElement )
  {
    this.constructionModel.setRoot(modelElement);
    ConstructionArea.notifyListeners(this.aRootSetHandles, modelElement.getElementId());
  }
}
ConstructionArea.prototype.getSetRootAction = function()
{
  return new ObjectMethodAction(this, "setRoot");
}
ConstructionArea.prototype.addRootSetHandle = function( action )
{
  if( action )
    this.aRootSetHandles.push(action);
}
ConstructionArea.prototype.addElement = function( modelElement )
{
  if( modelElement )
  {
    this.constructionModel.addElement(modelElement);
    ConstructionArea.notifyListeners(this.aElementAddedHandles, modelElement.getElementId());
  }
}
ConstructionArea.prototype.getAddElementAction = function()
{
  return new ObjectMethodAction(this, "addElement");
}
ConstructionArea.prototype.addElementAddedHandle = function( action )
{
  if( action )
    this.aElementAddedHandles.push(action);
}
ConstructionArea.prototype.removeElement = function( sElementId )
{
  if( this.constructionModel.root.getElementId() != sElementId )
  {
    this.constructionModel.removeElement(sElementId);
    ConstructionArea.notifyListeners(this.aElementRemovedHandles, sElementId);
  }
  else
  {
    ;
  }
}
ConstructionArea.prototype.getRemoveElementAction = function()
{
  return new ObjectMethodAction(this, "removeElement");
}
ConstructionArea.prototype.addElementRemovedHandle = function( action )
{
  if( action )
    this.aElementRemovedHandles.push(action);
}
ConstructionArea.prototype.findElement = function( sElementId )
{
  var modelelement = this.constructionModel.findElement(sElementId);
  ConstructionArea.notifyListeners(this.aElementFoundHandles, modelelement);
  return modelelement;
}
ConstructionArea.prototype.getFindElementAction = function()
{
  return new ObjectMethodAction(this, "findElement");
}
ConstructionArea.prototype.addElementFoundHandle = function( action )
{
  if( action )
    this.aElementFoundHandles.push(action);
}
ConstructionArea.prototype.createElement = function( sElementId, sTypename )
{
  var element = this.constructionModel.createElement(sElementId, sTypename);
  if( element )
    ConstructionArea.notifyListeners(this.aElementCreatedHandles, element);
  return element;
}
ConstructionArea.prototype.getCreateElementAction = function()
{
  return new ObjectMethodAction(this, "createElement");
}
ConstructionArea.prototype.addElementCreatedHandle = function( action )
{
  if( action )
    this.aElementCreatedHandles.push(action);
}
ConstructionArea.prototype.setImageSrc = function( imageSrc )
{
  if( imageSrc != null )
  {
    Browser.setImageSourceById(this.imageDomId, imageSrc);
    ConstructionArea.notifyListeners(this.aImageSrcSetHandles, imageSrc);
  }
}
ConstructionArea.prototype.getSetImageSrcAction = function()
{
  return new ObjectMethodAction(this, "setImageSrc");
}
ConstructionArea.prototype.addImageSrcSetHandle = function( action )
{
  if( action )
    this.aImageSrcSetHandles.push(action);
}
ConstructionArea.prototype.handleMouseClick = function( clientX, clientY )
{
  var pos = new Point(0, 0);
  Browser.getAbsPos(this.nImage, pos);
  var x = clientX + document.body.scrollLeft - pos.x;
  var y = clientY + document.body.scrollTop - pos.y;
  ConstructionArea.notifyListeners(this.aMouseClickHandles, x, y);
}
ConstructionArea.prototype.addMouseClickHandle = function( action )
{
  if( action )
    this.aMouseClickHandles.push(action);
}
ConstructionArea.prototype.isInScope = function( modelItem )
{
  return true;
}
ConstructionArea.prototype.assign = function( modelItem, x, y )
{
  var pos = new Point(0, 0);
  Browser.getAbsPos(this.nImage, pos);
  var ix = x + document.body.scrollLeft - pos.x;
  var iy = y + document.body.scrollTop - pos.y;
  ConstructionArea.notifyListeners(this.aMouseDropHandles, modelItem.getTypename(), modelItem.isRootType, ix, iy);
}
ConstructionArea.prototype.addMouseDropHandle = function( action )
{
  if( action )
    this.aMouseDropHandles.push(action);
}
ConstructionArea.notifyListeners = function( handles, param1, param2, param3, param4 )
{
  for( var i = 0; i < handles.length; i++ )
  {
    handles[i].invoke(param1, param2, param3, param4);
  }
}
ConstructionArea.createMouseClickEventHandler = function( me )
{
  me.mouseClickHandler = function( event )
  {
    if( !event )
      event = window.event;
    me.handleMouseClick(event.clientX, event.clientY);
  }
}
