/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
    <DEPENDS>ImagePushButton</DEPENDS>
</DEPENDENCIES>
*/
function DialogStoryRowUser(sId, oView)
{
   this.init(sId, oView);
}
DialogStoryRowUser.prototype.init = function(sId, oView, soundStoppedCallbackAction)
{
  this.sId = sId;
  this.view = oView;
  this.aStoryElementUser = new Array();
  this.aSoundBroker = new Array();
  this.aSingleRows = new Array();
  this.element = null;
}
DialogStoryRowUser.prototype.addStoryElement = function(oStoryElementUser, soundBroker)
{
  this.aStoryElementUser.push(oStoryElementUser),
  this.aSoundBroker.push(soundBroker);
}
DialogStoryRowUser.prototype.show = function()
{
  this.appendToDisplay(this.view.displayElement);
}
DialogStoryRowUser.prototype.hide = function()
{
  this.removeFromDisplay(this.view.displayElement);
}
DialogStoryRowUser.prototype.appendToDisplay = function(parentNode)
{
  if(!this.element)
  {
    this.buildHTMLRepresentation();
  }
  parentNode.appendChild(this.element);
  for(var i in this.aSingleRows)
  {
    var singleRow = this.aSingleRows[i];
    singleRow.appendToDisplay(this.element);
  }
}
DialogStoryRowUser.prototype.removeFromDisplay = function(parentNode)
{
  if(this.element)
  {
    Browser.removeNodeFromParent(this.element);
  }
}
DialogStoryRowUser.prototype.buildHTMLRepresentation = function()
{
  this.element = document.createElement('div');
  for(var i in this.aStoryElementUser)
  {
    var storyElementUser = this.aStoryElementUser[i];
    var singleRow = new DialogStorySingleRowUser(this.sId + "_" + i, storyElementUser, this.view);
    singleRow.setSoundBroker(this.aSoundBroker[i]);
    this.aSingleRows.push(singleRow);
  }
}
function DialogStorySingleRowUser(sId, oStoryElementUser, oView)
{
   this.init(sId, oStoryElementUser, oView);
}
DialogStorySingleRowUser.prototype.init = function(sId, oStoryElementUser, oView)
{
  this.sId = sId;
  this.oStoryElementUser = oStoryElementUser;
  this.view = oView;
  this.soundBroker = null;
  this.element = null;
  this.imageButtonElement = null;
  this.oImageButton = null;
  this.textElement = null;
  this.bTextHighlighted = false;
}
DialogStorySingleRowUser.prototype.setSoundBroker = function(soundBroker)
{
  this.soundBroker = soundBroker;
}
DialogStorySingleRowUser.prototype.appendToDisplay = function(parentNode)
{
  if(!this.element)
  {
    this.buildHTMLRepresentation();
  }
  parentNode.appendChild(this.element);
  if(this.imageButtonElement)
  {
    this.oImageButton = new ImagePushButton(
      this.imageButtonElement.id,
      this.view.sSoundPlayerNormalImageUrl,
      this.view.sSoundPlayerHighlightImageUrl,
      this.view.sSoundPlayerNormalImageUrl,
      this.view.sSoundPlayerHighlightImageUrl,
      true);
    if(this.soundBroker)
    {
      this.oImageButton.setPressedAction(this.soundBroker.getRequestSoundStartAction());
    }
  }
}
DialogStorySingleRowUser.prototype.removeFromDisplay = function(parentNode)
{
  if(this.element)
  {
    Browser.removeNodeFromParent(this.element);
  }
}
DialogStorySingleRowUser.prototype.buildHTMLRepresentation = function()
{
  var textString = '';
  if(this.view.sTextStyleClass)
  {
    textString = '<span id="' + this.sId + '_text" class="' + this.view.sTextStyleClass + '">' + this.oStoryElementUser.getText() + "</span>";
  }
  else
  {
    textString = '<span id="' + this.sId + '_text">' + this.oStoryElementUser.getText() + "</span>";
  }
  var rowDivString;
  if(this.view.sUserRowStyleClass)
  {
    rowDivString = '<div id="' + this.sId + '_rowdiv" class="' + this.view.sUserRowStyleClass + '"></div>';
  }
  else
  {
    rowDivString = '<div id="' + this.sId + '_rowdiv"></div>';
  }
  var divElement = document.createElement('div');
  divElement.style.position = 'relative';
  divElement.innerHTML = rowDivString;
  var rowDivElement = divElement.firstChild;
  rowDivElement.innerHTML = textString;
  this.textElement = rowDivElement.firstChild;
  if(this.soundBroker)
  {
    this.soundBroker.addSoundStartedHandle(this.getHighlightTextAction());
    this.soundBroker.addSoundStoppedHandle(this.getNormalTextAction());
  }
  if(this.view.bShowSoundPlayer)
  {
     this.imageButtonElement = document.createElement('img');
     this.imageButtonElement.id = this.sId + '_sounplayerimg';
     this.imageButtonElement.src = this.view.sSoundPlayerNormalImageUrl;
     this.imageButtonElement.width = this.view.nSoundPlayerImageWidth;
     this.imageButtonElement.height = this.view.nSoundPlayerImageHeight;
     this.imageButtonElement.style.verticalAlign = 'middle';
     rowDivElement.insertBefore(this.imageButtonElement, rowDivElement.firstChild);
  }
  this.element = divElement;
}
DialogStorySingleRowUser.prototype.highlightText = function()
{
  if(!this.bTextHighlighted)
  {
    if(this.view.sFgColorHighlight)
    {
      this.textElement.style.color = this.view.sFgColorHighlight;
    }
    if(this.view.sBgColorHighlight)
    {
      this.textElement.style.backgroundColor = this.view.sBgColorHighlight;
    }
    this.bTextHighlighted = true;
  }
}
DialogStorySingleRowUser.prototype.normalText = function()
{
  if(this.bTextHighlighted)
  {
    this.textElement.style.color = '';
    this.textElement.style.backgroundColor = '';
    this.bTextHighlighted = false;
  }
}
DialogStorySingleRowUser.prototype.getHighlightTextAction = function()
{
  return new ObjectMethodAction(this, "highlightText");
}
DialogStorySingleRowUser.prototype.getNormalTextAction = function()
{
  return new ObjectMethodAction(this, "normalText");
}
